using System;
using System.Collections;
using System.Collections.Specialized;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Syncfusion.Windows.Forms.Grid;
using System.Runtime.Serialization;

namespace TestGridCombobox
{
    public partial class Form1 : Form
    {
        const int Col1 = 1;
        const int Col2 = 2;
        const int Col3 = 3;
        const int Col4 = 4;
        StringCollection m_subChoices;
        StringCollection m_choices;
        int m_savedIndex = -1;
        ArrayList m_greyItems;

        public Form1()
        {
            InitializeComponent();

            m_greyItems = new ArrayList();
            m_greyItems.Add("SubTest2");

            m_subChoices = new StringCollection();
            m_subChoices.Add("SubTest1");
            m_subChoices.Add("SubTest2");
            m_subChoices.Add("SubTest3");
            m_subChoices.Add("SubTest4");

            m_choices = new StringCollection();
            m_choices.Add("Test1");
            m_choices.Add("Test2");
            m_choices.Add("Test3");
            m_choices.Add("Test4");
            this.gridControl1.ColCount = 3;
            this.gridControl1.CurrentCellCloseDropDown += new Syncfusion.Windows.Forms.PopupClosedEventHandler(gridControl1_CurrentCellCloseDropDown);
            this.gridControl1.CurrentCellShowingDropDown += new GridCurrentCellShowingDropDownEventHandler(gridControl1_CurrentCellShowingDropDown);
            this.gridControl1[2, Col3].CellType = "ComboBox";
            this.gridControl1[2, Col3].ChoiceList = m_choices;
        }

        void gridControl1_CurrentCellCloseDropDown(object sender, Syncfusion.Windows.Forms.PopupClosedEventArgs e)
        {
            GridControl gridControl = sender as GridControl;
			GridCurrentCell cc = gridControl.CurrentCell;
            if (cc.RowIndex == 2 && cc.ColIndex == Col3)
            {
                string filter = GetComboBoxText(gridControl);
                if (filter == "Test1")
                {
                  gridControl.ColCount = 4;
                    AddChoiceList(gridControl, 2, Col4, m_subChoices);
                }
                else if (filter == "Test2")
                { 
                    gridControl.ColCount = 4;
                    AddNewCellType(gridControl, 2, Col4, 64);
                }
            }
            // Will "gridControl[2, Col3].Text" always give correct value?? Or is there a way to
            // get the selected text in gridControl[2, Col3] correctly??
            else if (cc.RowIndex == 2 && cc.ColIndex == Col4 && gridControl[2, Col3].Text == "Test1")
            {
                //Show whats displayed in the First Combo box
                MessageBox.Show(gridControl[2, Col3].Text, "Selected text in Previous Column");
            }
            else
            {
                MessageBox.Show(gridControl[2, Col3].Text, "Selected text in Previous Column");
            }
            
        }

        public static String GetComboBoxText(object gridControl)
        {
            GridControl gridCtrl = (GridControl)gridControl;
            GridCurrentCell cc = null;
            GridCellRendererBase renderer = null;
            GridStyleInfo style = null;
            string ct = "";
            if (gridCtrl != null && gridCtrl.CurrentCell.HasCurrentCell)
            {
                cc = gridCtrl.CurrentCell;
                renderer = cc.Renderer;
                style = gridCtrl[cc.RowIndex, cc.ColIndex];

                ct = renderer.ControlText;
                if (renderer.HasControlText)
                    ct = renderer.ControlText;
            }
            return ct;
        }

        void AddChoiceList(GridControl gridControl, int rowIndex, int colIndex, StringCollection list)
        {
            gridControl[rowIndex, colIndex].CellType = "ComboBox";
            gridControl[rowIndex, colIndex].ChoiceList = list;
            gridControl[rowIndex, colIndex].ExclusiveChoiceList = true;
            gridControl[rowIndex, colIndex].Text = list[0];
            gridControl[rowIndex, colIndex].Font.Facename = "Tahoma";
            gridControl[rowIndex, colIndex].Font.Size = (float)8.25;
        }

        private void gridControl1_CurrentCellShowingDropDown(object sender, GridCurrentCellShowingDropDownEventArgs e)
        {
            GridControl gridControl = sender as GridControl;
            GridCurrentCell cc = gridControl.CurrentCell;
            if (cc.Renderer is GridComboBoxCellRenderer)
            {
                GridComboBoxCellRenderer cr = cc.Renderer as GridComboBoxCellRenderer;
                if (cc.RowIndex == 2 && cc.ColIndex == Col4 && gridControl[2, Col3].Text == "Test2")
                {
                    cr.ListBoxPart.DrawMode = DrawMode.OwnerDrawVariable;
                    cr.ListBoxPart.DrawItem -= new DrawItemEventHandler(ListBoxPart_DrawItem);
                    cr.ListBoxPart.DrawItem += new DrawItemEventHandler(ListBoxPart_DrawItem);
                }
                else
                    cr.ListBoxPart.DrawMode = DrawMode.Normal;
            }
        }

        private void ListBoxPart_DrawItem(object sender, DrawItemEventArgs e)
        {
            ListBox list = sender as ListBox;
            StringFormat format = new StringFormat();
            format.LineAlignment = StringAlignment.Center;
            format.Alignment = StringAlignment.Center;

            RectangleF rect = new RectangleF(e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height);

            if (m_greyItems.Contains(list.Items[e.Index].ToString()))
            {
                e.Graphics.FillRectangle(Brushes.LightGray, e.Bounds);
                e.Graphics.DrawString(list.Items[e.Index].ToString(), list.Font, new SolidBrush(list.ForeColor), rect, format);
            }

            else
            {
                if (list.SelectedIndex == e.Index)
                {
                    if (m_savedIndex != -1 && m_savedIndex != list.SelectedIndex)
                        list.Invalidate(list.GetItemRectangle(m_savedIndex));
                    m_savedIndex = list.SelectedIndex;
                    e.Graphics.FillRectangle(Brushes.MidnightBlue, e.Bounds);
                    e.Graphics.DrawString(list.Items[e.Index].ToString(), list.Font, new SolidBrush(list.BackColor), rect, format);
                }
                else
                    e.Graphics.DrawString(list.Items[e.Index].ToString(), list.Font, new SolidBrush(list.ForeColor), rect, format);
            }
        }
        /// <summary>
        /// Adds the newly created cell type to the specified row Index.
        /// </summary>
        /// <param name="gridControl"></param>
        /// <param name="rowIndex"></param>
        /// <param name="colIndex"></param>
        /// <param name="bitSize"></param>
        void AddNewCellType(GridControl gridControl, int rowIndex, int colIndex, int bitSize)
        {
            if (rowIndex <= gridControl.RowCount && colIndex <= gridControl.ColCount)
            {
                if (!gridControl.CellModels.ContainsKey("NewType"))
                    gridControl.CellModels.Add("NewType", new NewCellModel(gridControl.Model));
                GridStyleInfo cell = gridControl[rowIndex, colIndex];
                cell.CellType = "NewType";
            }
        }
    }
    /// <summary>
    /// This is a New CellModel that has to appear in the dropdown
    /// </summary>
    public class NewCellModel : GridDropDownGridCellModel
    {
        private GridControl newGrid;

        internal GridControl EmbeddedGrid { get { return newGrid; } }

        protected NewCellModel(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
        }

        public NewCellModel(GridModel grid)
            : base(grid)
        {
            newGrid = new GridControl();
            newGrid.RowCount = 2;
            newGrid.ColCount = 1;
            newGrid.Properties.RowHeaders = false;
            newGrid.Properties.ColHeaders = false;
            newGrid.CausesValidation = false;
            newGrid.ThemesEnabled = true;

            newGrid.ColWidths[1] = 70;
            Rectangle rect = newGrid.RangeInfoToRectangle(GridRangeInfo.Cells(1, 1, 2, 1));
            newGrid.Size = new Size(rect.Width, rect.Height);
        }

        public override GridCellRendererBase CreateRenderer(GridControlBase control)
        {
            return new NewCellRenderer(control, this);
        }

        public int WidthTotal
        {
            get
            {
                return newGrid.ColWidths.GetTotal(1, newGrid.ColCount) + 16;
            }
        }
        public int HeightTotal
        {
            get
            {
                return newGrid.RowHeights.GetTotal(0, newGrid.RowCount);
            }
        }
    }
    /// <summary>
    /// Cell Renderer for New Cell Model
    /// </summary>
    public class NewCellRenderer : GridDropDownGridCellRenderer
    {
        GridControlBase m_grid;

        public NewCellRenderer(GridControlBase grid, GridCellModelBase cellModel)
            : base(grid, cellModel)
        {
            DropDownButton = new GridCellComboBoxButton(this);
            this.m_grid = null;
        }

        protected override void OnInitialize(int rowIndex, int colIndex)
        {
            this.m_grid = ((NewCellModel)this.Model).EmbeddedGrid;
            this.DropDownContainer.Size = new Size(150, 43);
            base.OnInitialize(rowIndex, colIndex);
        }

        protected override Control CreateInnerControl(out GridControlBase grid)
        {
            grid = this.m_grid;
            m_grid.CausesValidation = false;
            return m_grid;
        }
    }
}